@@ -22,6 +22,11 @@
  * SOFTWARE.
  */
 
+
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include<windows.h>
 #include <wchar.h>
 #define _CRT_RAND_S
@@ -31,6 +36,7 @@
 #include "ms-srvsvc.h"
 #include "msrpc.h"
 #include "afsd.h"
+#include "smb.h"
 #include <WINNT/afsreg.h>
 #define AFS_VERSION_STRINGS
 #include "AFS_component_version_number.h"
@@ -98,6 +104,7 @@ NET_API_STATUS NetrConnectionEnum(
     /* [out] */ DWORD *TotalEntries,
     /* [unique][out][in] */ DWORD *ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrConnectionEnum not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -110,6 +117,7 @@ NET_API_STATUS NetrFileEnum(
     /* [out] */ DWORD *TotalEntries,
     /* [unique][out][in] */ DWORD *ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrFileEnum not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -119,6 +127,7 @@ NET_API_STATUS NetrFileGetInfo(
     /* [in] */ DWORD Level,
     /* [switch_is][out] */ LPFILE_INFO InfoStruct)
 {
+    osi_Log0(afsd_logp, "NetrFileGetInfo not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -126,6 +135,7 @@ NET_API_STATUS NetrFileClose(
     /* [unique][string][in] */ SRVSVC_HANDLE ServerName,
     /* [in] */ DWORD FileId)
 {
+    osi_Log0(afsd_logp, "NetrFileClose not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -138,6 +148,7 @@ NET_API_STATUS NetrSessionEnum(
     /* [out] */ DWORD *TotalEntries,
     /* [unique][out][in] */ DWORD *ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrSessionEnum not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -146,6 +157,7 @@ NET_API_STATUS NetrSessionDel(
     /* [unique][string][in] */ WCHAR *ClientName,
     /* [unique][string][in] */ WCHAR *UserName)
 {
+    osi_Log0(afsd_logp, "NetrSessionDel not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -155,6 +167,7 @@ NET_API_STATUS NetrShareAdd(
     /* [switch_is][in] */ LPSHARE_INFO InfoStruct,
     /* [unique][out][in] */ DWORD *ParmErr)
 {
+    osi_Log0(afsd_logp, "NetrShareAdd not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -298,10 +311,11 @@ NetrIntGenerateSharePath(wchar_t *ServerName, cm_fid_t *fidp)
     else
         wcellname = L"";
 
+    for ( s=ServerName; *s == '\\' || *s == '/'; s++);
     hr = StringCchPrintfW( remark,
                            sizeof(remark)/sizeof(remark[0]),
-                           L"\\%s\\%s\\%u.%u.%u",
-                           ServerName, wcellname,
+                           L"\\\\%s\\%s\\%u.%u.%u",
+                           s, wcellname,
                            fidp->volume,
                            fidp->vnode, fidp->unique);
     if (hr == S_OK)
@@ -328,14 +342,31 @@ static DWORD         shareEnum_next_handle=1;
 void
 RPC_SRVSVC_Init(void)
 {
+#if _MSC_VER >= 1400
+    int hasRand_s = -1;
+    OSVERSIONINFO osInfo;
+
+    osInfo.dwOSVersionInfoSize = sizeof(osInfo);
+    if (GetVersionEx(&osInfo)) {
+        if ((osInfo.dwMajorVersion > 5) ||
+             (osInfo.dwMajorVersion == 5) && (osInfo.dwMinorVersion >= 1))
+            hasRand_s = 1;
+        else
+            hasRand_s = 0;
+    }
+#endif
+
     lock_InitializeMutex(&shareEnum_mx, "NetrShareEnum", 0);
     shareEnumQ = NULL;
-#if _MSC_VER < 1400
-    srand((unsigned) time( NULL ));
-    shareEnum_next_handle = rand();
-#else
-    rand_s(&shareEnum_next_handle);
+#if _MSC_VER >= 1400
+    if (hasRand_s > 0) {
+        rand_s(&shareEnum_next_handle);
+    } else
 #endif
+    {
+        srand((unsigned) time( NULL ));
+        shareEnum_next_handle = rand();
+    }
 }
 
 void
@@ -429,7 +460,7 @@ NET_API_STATUS NetrShareEnum(
 {
     cm_direnum_t *enump = NULL;
     cm_direnum_entry_t * entryp = NULL;
-    cm_scache_t * dscp = cm_data.rootSCachep;
+    cm_scache_t * dscp;
     cm_user_t *userp = MSRPC_GetCmUser();
     cm_req_t      req;
     int           stopnow = 0;
@@ -440,8 +471,12 @@ NET_API_STATUS NetrShareEnum(
     size_t        space_available = 0;
     afs_uint32    first_entry = 1;
 
+    osi_Log1(afsd_logp, "NetrShareEnum level %u", InfoStruct->Level);
+
     cm_InitReq(&req);
 
+    dscp = cm_RootSCachep(userp, &req);
+
     RPC_SRVSVC_ShareEnumAgeCheck();
 
     /* We only support one server name so ignore 'ServerName'. */
@@ -490,7 +525,8 @@ NET_API_STATUS NetrShareEnum(
     if (ResumeHandle == NULL || *ResumeHandle == 0) {
         cm_dirOp_t    dirop;
 
-        code = cm_BeginDirOp(dscp, userp, &req, CM_DIRLOCK_READ, &dirop);
+        code = cm_BeginDirOp(dscp, userp, &req, CM_DIRLOCK_READ,
+                             CM_DIROP_FLAG_NONE, &dirop);
         if (code == 0) {
             code = cm_BPlusDirEnumerate(dscp, userp, &req, TRUE, NULL, TRUE, &enump);
             if (code) {
@@ -552,17 +588,17 @@ NET_API_STATUS NetrShareEnum(
     case 2:
         if (space_limited)
             space_available -= sizeof(InfoStruct->ShareInfo.Level2);
-        InfoStruct->ShareInfo.Level2->Buffer = calloc((enump->count - enump->next), sizeof(SHARE_INFO_2));
+        InfoStruct->ShareInfo.Level2->Buffer = MIDL_user_allocate((enump->count - enump->next) * sizeof(SHARE_INFO_2));
         break;
     case 1:
         if (space_limited)
             space_available -= sizeof(InfoStruct->ShareInfo.Level1);
-        InfoStruct->ShareInfo.Level1->Buffer = calloc((enump->count - enump->next), sizeof(SHARE_INFO_1));
+        InfoStruct->ShareInfo.Level1->Buffer = MIDL_user_allocate((enump->count - enump->next) * sizeof(SHARE_INFO_1));
         break;
     case 0:
         if (space_limited)
             space_available -= sizeof(InfoStruct->ShareInfo.Level0);
-        InfoStruct->ShareInfo.Level0->Buffer = calloc((enump->count - enump->next), sizeof(SHARE_INFO_0));
+        InfoStruct->ShareInfo.Level0->Buffer = MIDL_user_allocate((enump->count - enump->next) * sizeof(SHARE_INFO_0));
         break;
     }
 
@@ -592,7 +628,7 @@ NET_API_STATUS NetrShareEnum(
             continue;
         }
 
-        cm_GetSCache(&entryp->fid, &scp, userp, &req);
+        cm_GetSCache(&entryp->fid, NULL, &scp, userp, &req);
 
         switch (InfoStruct->Level) {
         case 2:
@@ -682,7 +718,7 @@ NET_API_STATUS NetrShareGetInfo(
     /* [in] */ DWORD Level,
     /* [switch_is][out] */ LPSHARE_INFO InfoStruct)
 {
-    cm_scache_t * dscp = cm_data.rootSCachep;
+    cm_scache_t * dscp;
     cm_scache_t * scp = NULL;
     cm_user_t *userp = MSRPC_GetCmUser();
     cm_req_t      req;
@@ -693,18 +729,22 @@ NET_API_STATUS NetrShareGetInfo(
     clientchar_t pathstr[CELL_MAXNAMELEN + VL_MAXNAMELEN + 1 + CM_PREFIX_VOL_CCH];
     DWORD  cblen;
 
+    osi_Log1(afsd_logp, "NetrShareGetInfo level %u", Level);
+
     cm_InitReq(&req);
 
+    dscp = cm_RootSCachep(userp, &req);
+
     /* Allocate the memory for the response */
     switch (Level) {
     case 2:
-        InfoStruct->ShareInfo2 = calloc(1, sizeof(SHARE_INFO_2));
+        InfoStruct->ShareInfo2 = MIDL_user_allocate(sizeof(SHARE_INFO_2));
         break;
     case 1:
-        InfoStruct->ShareInfo1 = calloc(1, sizeof(SHARE_INFO_1));
+        InfoStruct->ShareInfo1 = MIDL_user_allocate(sizeof(SHARE_INFO_1));
         break;
     case 0:
-        InfoStruct->ShareInfo0 = calloc(1, sizeof(SHARE_INFO_0));
+        InfoStruct->ShareInfo0 = MIDL_user_allocate(sizeof(SHARE_INFO_0));
         break;
     }
 
@@ -800,9 +840,11 @@ NET_API_STATUS NetrShareGetInfo(
 
             if (cblen) {
                 code = cm_NameI(dscp, pathstr, CM_FLAG_FOLLOW, userp, NULL, &req, &scp);
-                if (code && code != CM_ERROR_NOACCESS)
+                if (code == CM_ERROR_NOSUCHFILE ||
+                    code == CM_ERROR_NOSUCHPATH ||
+                    code == CM_ERROR_BPLUS_NOMATCH)
                     code = cm_NameI(dscp, pathstr, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
-                                      userp, NULL, &req, &scp);
+                                    userp, NULL, &req, &scp);
             }
         }
     }
@@ -863,6 +905,7 @@ NET_API_STATUS NetrShareSetInfo(
     /* [switch_is][in] */ LPSHARE_INFO ShareInfo,
     /* [unique][out][in] */ DWORD *ParmErr)
 {
+    osi_Log0(afsd_logp, "NetrShareSetInfo not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -871,6 +914,7 @@ NET_API_STATUS NetrShareDel(
     /* [string][in] */ WCHAR *NetName,
     /* [in] */ DWORD Reserved)
 {
+    osi_Log0(afsd_logp, "NetrShareDel not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -879,6 +923,7 @@ NET_API_STATUS NetrShareDelSticky(
     /* [string][in] */ WCHAR *NetName,
     /* [in] */ DWORD Reserved)
 {
+    osi_Log0(afsd_logp, "NetrShareDelSticky not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -887,6 +932,7 @@ NET_API_STATUS NetrShareCheck(
     /* [string][in] */ WCHAR *Device,
     /* [out] */ DWORD *Type)
 {
+    osi_Log0(afsd_logp, "NetrShareCheck not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -895,24 +941,27 @@ NET_API_STATUS NetrServerGetInfo(
     /* [in] */ DWORD Level,
     /* [switch_is][out] */ LPSERVER_INFO InfoStruct)
 {
+    wchar_t *s;
+
+    osi_Log1(afsd_logp, "NetrServerGetInfo level %u", Level);
     /*
-    * How much space do we need and do we have that much room?
-    * For now, just assume we can return everything in one shot
-    * because the reality is that in this function call we do
-    * not know the max size of the RPC response.
-    */
+     * How much space do we need and do we have that much room?
+     * For now, just assume we can return everything in one shot
+     * because the reality is that in this function call we do
+     * not know the max size of the RPC response.
+     */
     switch (Level) {
     case 103:
-        InfoStruct->ServerInfo103 = calloc(1, sizeof(SERVER_INFO_103));
+        InfoStruct->ServerInfo103 = MIDL_user_allocate(sizeof(SERVER_INFO_103));
         break;
     case 102:
-        InfoStruct->ServerInfo102 = calloc(1, sizeof(SERVER_INFO_102));
+        InfoStruct->ServerInfo102 = MIDL_user_allocate(sizeof(SERVER_INFO_102));
         break;
     case 101:
-        InfoStruct->ServerInfo101 = calloc(1, sizeof(SERVER_INFO_101));
+        InfoStruct->ServerInfo101 = MIDL_user_allocate(sizeof(SERVER_INFO_101));
         break;
     case 100:
-        InfoStruct->ServerInfo100 = calloc(1, sizeof(SERVER_INFO_100));
+        InfoStruct->ServerInfo100 = MIDL_user_allocate(sizeof(SERVER_INFO_100));
         break;
     }
 
@@ -920,6 +969,12 @@ NET_API_STATUS NetrServerGetInfo(
         return ERROR_NOT_ENOUGH_MEMORY;
     }
 
+    /*
+     * Remove any leading slashes since they are not part of the
+     * server name.
+     */
+    for ( s=ServerName; *s == '\\' || *s == '/'; s++);
+
     switch (Level) {
     case 103:
         InfoStruct->ServerInfo103->sv103_capabilities = 0;
@@ -936,13 +991,13 @@ NET_API_STATUS NetrServerGetInfo(
     case 101:
         InfoStruct->ServerInfo101->sv101_version_major = AFSPRODUCT_VERSION_MAJOR;
         InfoStruct->ServerInfo101->sv101_version_minor = AFSPRODUCT_VERSION_MINOR;
-        InfoStruct->ServerInfo101->sv101_type = SV_TYPE_SERVER_UNIX;
+        InfoStruct->ServerInfo101->sv101_type = SV_TYPE_WORKSTATION | SV_TYPE_SERVER | SV_TYPE_SERVER_UNIX;
         InfoStruct->ServerInfo101->sv101_comment = wcsdup(wAFSVersion);
         /* fall-through */
     case 100:
         InfoStruct->ServerInfo100->sv100_platform_id = SV_PLATFORM_ID_AFS;
         /* The Netbios Name */
-        InfoStruct->ServerInfo100->sv100_name = wcsdup(ServerName);
+        InfoStruct->ServerInfo100->sv100_name = _wcsupr(wcsdup(s));
         return 0;
     case 502:
     case 503:
@@ -1003,6 +1058,7 @@ NET_API_STATUS NetrServerSetInfo(
     /* [switch_is][in] */ LPSERVER_INFO ServerInfo,
     /* [unique][out][in] */ DWORD *ParmErr)
 {
+    osi_Log0(afsd_logp, "NetrServerSetInfo not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1014,6 +1070,7 @@ NET_API_STATUS NetrServerDiskEnum(
     /* [out] */ DWORD *TotalEntries,
     /* [unique][out][in] */ DWORD *ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrServerDiskEnum not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1024,6 +1081,7 @@ NET_API_STATUS NetrServerStatisticsGet(
     /* [in] */ DWORD Options,
     /* [out] */ LPSTAT_SERVER_0 *InfoStruct)
 {
+    osi_Log0(afsd_logp, "NetrServerStatisticsGet not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1032,6 +1090,7 @@ NET_API_STATUS NetrServerTransportAdd(
     /* [in] */ DWORD Level,
     /* [in] */ LPSERVER_TRANSPORT_INFO_0 Buffer)
 {
+    osi_Log0(afsd_logp, "NetrServerTransportAdd not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1042,6 +1101,7 @@ NET_API_STATUS NetrServerTransportEnum(
     /* [out] */ DWORD *TotalEntries,
     /* [unique][out][in] */ DWORD *ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrServerTransportEnum not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1050,6 +1110,7 @@ NET_API_STATUS NetrServerTransportDel(
     /* [in] */ DWORD Level,
     /* [in] */ LPSERVER_TRANSPORT_INFO_0 Buffer)
 {
+    osi_Log0(afsd_logp, "NetrServerTransportDel not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1057,6 +1118,7 @@ NET_API_STATUS NetrRemoteTOD(
     /* [unique][string][in] */ SRVSVC_HANDLE ServerName,
     /* [out] */ LPTIME_OF_DAY_INFO *BufferPtr)
 {
+    osi_Log0(afsd_logp, "NetrRemoteTOD not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1066,6 +1128,7 @@ NET_API_STATUS NetprPathType(
     /* [out] */ DWORD *PathType,
     /* [in] */ DWORD Flags)
 {
+    osi_Log0(afsd_logp, "NetprPathType not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1078,6 +1141,7 @@ NET_API_STATUS NetprPathCanonicalize(
     /* [out][in] */ DWORD *PathType,
     /* [in] */ DWORD Flags)
 {
+    osi_Log0(afsd_logp, "NetprPathCanonicalize not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1088,6 +1152,7 @@ long NetprPathCompare(
     /* [in] */ DWORD PathType,
     /* [in] */ DWORD Flags)
 {
+    osi_Log0(afsd_logp, "NetprPathCompare not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1097,6 +1162,7 @@ NET_API_STATUS NetprNameValidate(
     /* [in] */ DWORD NameType,
     /* [in] */ DWORD Flags)
 {
+    osi_Log0(afsd_logp, "NetprNameValidate not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1108,6 +1174,7 @@ NET_API_STATUS NetprNameCanonicalize(
     /* [in] */ DWORD NameType,
     /* [in] */ DWORD Flags)
 {
+    osi_Log0(afsd_logp, "NetprNameCanonicalize not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1118,6 +1185,7 @@ long NetprNameCompare(
     /* [in] */ DWORD NameType,
     /* [in] */ DWORD Flags)
 {
+    osi_Log0(afsd_logp, "NetprNameCompare not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1128,6 +1196,7 @@ NET_API_STATUS NetrShareEnumSticky(
     /* [out] */ DWORD *TotalEntries,
     /* [unique][out][in] */ DWORD *ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrShareEnumSticky not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1137,12 +1206,14 @@ NET_API_STATUS NetrShareDelStart(
     /* [in] */ DWORD Reserved,
     /* [out] */ PSHARE_DEL_HANDLE ContextHandle)
 {
+    osi_Log0(afsd_logp, "NetrShareDelStart not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
 NET_API_STATUS NetrShareDelCommit(
     /* [out][in] */ PSHARE_DEL_HANDLE ContextHandle)
 {
+    osi_Log0(afsd_logp, "NetrShareDelCommit not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1153,6 +1224,7 @@ DWORD NetrpGetFileSecurity(
     /* [in] */ SECURITY_INFORMATION RequestedInformation,
     /* [out] */ PADT_SECURITY_DESCRIPTOR *SecurityDescriptor)
 {
+    osi_Log0(afsd_logp, "NetprGetFileSecurity not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1163,6 +1235,7 @@ DWORD NetrpSetFileSecurity(
     /* [in] */ SECURITY_INFORMATION SecurityInformation,
     /* [in] */ PADT_SECURITY_DESCRIPTOR SecurityDescriptor)
 {
+    osi_Log0(afsd_logp, "NetprSetFileSecurity not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1171,6 +1244,7 @@ NET_API_STATUS NetrServerTransportAddEx(
     /* [in] */ DWORD Level,
     /* [switch_is][in] */ LPTRANSPORT_INFO Buffer)
 {
+    osi_Log0(afsd_logp, "NetrServerTransportAddEx not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1178,6 +1252,7 @@ NET_API_STATUS NetrDfsGetVersion(
     /* [unique][string][in] */ SRVSVC_HANDLE ServerName,
     /* [out] */ DWORD *Version)
 {
+    osi_Log0(afsd_logp, "NetrDfsGetVersion not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1190,6 +1265,7 @@ NET_API_STATUS NetrDfsCreateLocalPartition(
     /* [in] */ LPNET_DFS_ENTRY_ID_CONTAINER RelationInfo,
     /* [in] */ int Force)
 {
+    osi_Log0(afsd_logp, "NetrDfsCreateLocalPartition not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1198,6 +1274,7 @@ NET_API_STATUS NetrDfsDeleteLocalPartition(
     /* [in] */ GUID *Uid,
     /* [string][in] */ WCHAR *Prefix)
 {
+    osi_Log0(afsd_logp, "NetrDfsDeleteLocalPartition not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1207,6 +1284,7 @@ NET_API_STATUS NetrDfsSetLocalVolumeState(
     /* [string][in] */ WCHAR *Prefix,
     /* [in] */ unsigned long State)
 {
+    osi_Log0(afsd_logp, "NetrDfsSetLocalVolumeState not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1218,6 +1296,7 @@ NET_API_STATUS NetrDfsCreateExitPoint(
     /* [range][in] */ DWORD ShortPrefixLen,
     /* [size_is][string][out] */ WCHAR *ShortPrefix)
 {
+    osi_Log0(afsd_logp, "NetrDfsCreateExitPoint not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1227,6 +1306,7 @@ NET_API_STATUS NetrDfsDeleteExitPoint(
     /* [string][in] */ WCHAR *Prefix,
     /* [in] */ unsigned long Type)
 {
+    osi_Log0(afsd_logp, "NetrDfsDeleteExitPoint not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1235,6 +1315,7 @@ NET_API_STATUS NetrDfsModifyPrefix(
     /* [in] */ GUID *Uid,
     /* [string][in] */ WCHAR *Prefix)
 {
+    osi_Log0(afsd_logp, "NetrDfsModifyPrefix not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1249,6 +1330,7 @@ NET_API_STATUS NetrDfsFixLocalVolume(
     /* [in] */ LPNET_DFS_ENTRY_ID_CONTAINER RelationInfo,
     /* [in] */ unsigned long CreateDisposition)
 {
+    osi_Log0(afsd_logp, "NetrDfsFixLocalVolume not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1256,6 +1338,7 @@ NET_API_STATUS NetrDfsManagerReportSiteInfo(
     /* [unique][string][in] */ SRVSVC_HANDLE ServerName,
     /* [unique][out][in] */ LPDFS_SITELIST_INFO *ppSiteInfo)
 {
+    osi_Log0(afsd_logp, "NetrDfsManagerReportSiteInfo not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1264,6 +1347,7 @@ NET_API_STATUS NetrServerTransportDelEx(
     /* [in] */ DWORD Level,
     /* [switch_is][in] */ LPTRANSPORT_INFO Buffer)
 {
+    osi_Log0(afsd_logp, "NetrServerTransportDelEx not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1272,6 +1356,7 @@ NET_API_STATUS NetrServerAliasAdd(
     /* [in] */ DWORD Level,
     /* [switch_is][in] */ LPSERVER_ALIAS_INFO InfoStruct)
 {
+    osi_Log0(afsd_logp, "NetrServerAliasAdd not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1282,6 +1367,7 @@ NET_API_STATUS NetrServerAliasEnum(
     /* [out] */ LPDWORD TotalEntries,
     /* [unique][out][in] */ LPDWORD ResumeHandle)
 {
+    osi_Log0(afsd_logp, "NetrServerAliasEnum not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1290,6 +1376,7 @@ NET_API_STATUS NetrServerAliasDel(
     /* [in] */ DWORD Level,
     /* [switch_is][in] */ LPSERVER_ALIAS_INFO InfoStruct)
 {
+    osi_Log0(afsd_logp, "NetrServerAliasDel not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
@@ -1298,6 +1385,7 @@ NET_API_STATUS NetrShareDelEx(
     /* [in] */ DWORD Level,
     /* [switch_is][in] */ LPSHARE_INFO ShareInfo)
 {
+    osi_Log0(afsd_logp, "NetrShareDelEx not supported");
     return ERROR_NOT_SUPPORTED;
 }
 
